import os
import shutil

from Scweet.scweet import scrape
import pandas as pd


def get_tweets_csv(user, start_date, end_date):
    '''
    Downloads CSV to an output directory generated within the current working directory.

    Args:
        user (str): Twitter username to emulate.
        start_date (str): Date to start scraping tweets from (YYYY-MM-DD)
        end_date (str): Date to end scraping tweets at (YYYY-MM-DD)

    Returns:
        tweets_df (Pandas Dataframe): Dataframe of downloaded tweets

    Raises:
        OSError: If an error occurs when trying to delete the CSV directory
    '''

    data = scrape(from_account=user, since=start_date, until=end_date)  # tweet download query

    # generates filepath for output tweets
    out_filename = '_'.join([user,start_date,end_date])
    out_filepath = os.path.join(os.getcwd(), "outputs", out_filename+'.csv')

    tweets_df = pd.read_csv(out_filepath)  # reads CSV as dataframe

    try:
        shutil.rmtree(os.path.join(os.getcwd(), "outputs"))  # deletes output directory and the file within it
    except OSError as e:
        print("Error: %s - %s." % (e.filename, e.strerror))

    return tweets_df




import os
import shutil

from datetime import datetime
import gpt_2_simple as gpt2

from . import download_tweets
from . import preprocessor


def train_gpt_2(out_tweets):
    '''
    Trains GPT-2 model and generates tweets to a file in the working directory.

    Args:
        out_tweets (int): Number of tweets to generate. 1000 by default.

    Returns:
        None

    Raises:
        OSError: If an error occurs when trying to delete the directories
    '''

    # adapts some hyperparameters based on file size
    if (os.path.getsize("tweets_file.txt") >= 1000000):
        model_name = "355M"
        lr = 1e-4
    else:
        model_name = "124M"
        lr = 1e-5

    print("Downloading specified model...")
    gpt2.download_gpt2(model_name=model_name)  # downloads specified gpt model

    sess = gpt2.start_tf_sess()  # starts tensorflow session

    print("Finetuning model...")
    # finetunes gpt-2
    gpt2.finetune(sess,
                dataset="tweets_file.txt",
                model_name=model_name,
                steps=2000,
                learning_rate=lr,
                restore_from="fresh",
    )
                
    gen_file = 'gpt2_gentext_{:%Y%m%d_%H%M%S}.txt'.format(datetime.utcnow())  # timestamped text file

    # generates output file with tweets
    gpt2.generate_to_file(sess,
                      destination_path=gen_file,  # output file destination
                      length=200,
                      temperature=1.0,
                      top_p=0.9,
                      prefix='<|startoftext|>',  # start tag
                      truncate='<|endoftext|>',  # end tag
                      include_prefix=False,
                      nsamples=out_tweets,  # number of tweets to generate
                      batch_size=20,
    )

    try:
        os.remove("tweets_file.txt")  # deletes downloaded tweets
        shutil.rmtree("models")  #  deletes downloaded model
        shutil.rmtree("checkpoint")  # deletes trained model
        shutil.rmtree("samples")  # deletes samples folder
    except OSError as e:
        print("Error: %s - %s." % (e.filename, e.strerror))


def emulate_user(user, start_date, end_date, out_tweets=1000):
    '''
    Args:
        user (str): Username for account to scrape for tweets
        start_date (str): Date to start scraping tweets from (YYYY-MM-DD)
        end_date (str): Date to end scraping tweets at (YYYY-MM-DD)
        out_tweets (int): Number of tweets to generate
    
    Returns:
        None
    '''
    
    delimiter = "=========================="

    print(f"{delimiter}\nDownloading Tweets...\n{delimiter}")
    tweets_df = download_tweets.get_tweets_csv(user, start_date, end_date)
    
    print(f"{delimiter}\nCleaning Tweets...\n{delimiter}")
    tweets_df = preprocessor.preprocessor(tweets_df)
    preprocessor.formatter(tweets_df)

    print(f"{delimiter}\nEmulating User...\n{delimiter}")
    train_gpt_2(out_tweets)

    print("All done! You may now find the output CSV in your working directory.")



def delimiter_cleaner(split_string):
    '''
    Removes delimiters from tweets.

    Args:
        split_string (str): Tweet to split

    Returns:
        If downloaded tweet has no delimiters:
            split_string (str): Original tweet
        If downloaded tweet has delimiters:
            split_string_clean (str): Tweet with delimiters cleaned
    '''
    if isinstance(split_string, list):  # if there was a delimiter in the text
        split_string_clean = []
        for seg in split_string:  # iterates parts segmented by the delimiter
            if (seg.isnumeric()):  # if segment contains only numbers
                pass
            elif seg == "Replying to ":  # if segment is replying template text
                pass
            elif seg.startswith("@"):  # if segment contains only a username
                pass
            else:
                split_string_clean.append(seg)

        split_string_clean = ''.join(split_string_clean)  # concatenates list items into single string
            
        return split_string_clean
    else:
        return split_string


def preprocessor(tweets_df):
    '''
    Cleans tweets dataframe.

    Args:
        tweets_df (Pandas Dataframe): Downloaded tweets

    Returns:
        tweets_df_clean (Pandas Dataframe): Cleaned downloaded tweets
    '''
    
    # removes entries with nan in tweets column
    tweets_df_clean = tweets_df[tweets_df['Embedded_text'].notna()]

    # splits tweets into segments separated by the \n delimiter
    tweets_df_clean["Embedded_text_clean"] = tweets_df_clean['Embedded_text'].str.split('\n')

    # applies delimiter cleaning function and pushes output to new column
    tweets_df_clean["Embedded_text_clean"] = tweets_df_clean["Embedded_text_clean"].apply(delimiter_cleaner)

    # removes tweets with a length less than 5 characters
    tweets_df_clean = tweets_df_clean[tweets_df_clean['Embedded_text_clean'].str.len()>=5]

    return tweets_df_clean


def formatter(tweets_df):
    '''
    Writes formatted tweets to a text file.

    Args:
        tweets_df (Pandas Dataframe): Downloaded tweets

    Returns:
        None
    '''

    with open("tweets_file.txt", 'w') as tweets_file:
        for t in tweets_df:
            tweets_file.write('<|startoftext|>'+t+'<|endoftext|>')






from setuptools import setup
from os import path

DIR = path.dirname(path.abspath(__file__))
INSTALL_PACKAGES = open(path.join(DIR, 'requirements.txt')).read().splitlines()

with open(path.join(DIR, 'README.md')) as f:
    README = f.read()

setup(
    name='twitter_user_emulator',
    packages=['twitter_user_emulator'],
    description="Twitter-User Emulating Tweet Generator",
    long_description=README,
    long_description_content_type='text/markdown',
    install_requires=INSTALL_PACKAGES,
    version='0.0.2',
    url='https://github.com/equ1/twitter_user_emulator',
    author='Rushat Rai',
    author_email='rushatrai@gmail.com',
    keywords=['deep-learning', 'gpt-2', 'twitter'],
    include_package_data=True,
    python_requires='>=3.6',
    license="MIT",
)
